@@ -5,7 +5,6 @@
 
 import os
 import sys
-import traceback
 
 from gunicorn import util
 from gunicorn.app.base import Application
@@ -23,3 +22,11 @@ class WSGIApplication(Application):
 
     def load(self):
         return util.import_app(self.app_uri)
+
+def run():
+    """\
+    The ``gunicorn`` command line runner for launcing Gunicorn with
+    generic WSGI applications.
+    """
+    from gunicorn.app.wsgiapp import WSGIApplication
+    WSGIApplication("%prog [OPTIONS] APP_MODULE").run()
\ No newline at end of file
